from dgredis import RedisConfig, RedisClient


def test_auto_get_dict():
    rconf = RedisConfig(
        host='localhost',
        port=6379,
        db=1,
        client_name='dgredis_test'
    )
    with RedisClient(cfg=rconf) as client:
        client.set('test_json', {'value': 'test'}, ttl=60)
        res = client.get('test_json')
        assert res and res.get('value') == 'test'
        assert isinstance(res, dict)


def test_auto_get_list():
    rconf = RedisConfig(
        host='localhost',
        port=6379,
        db=1,
        client_name='dgredis_test'
    )
    with RedisClient(cfg=rconf) as client:
        client.set('test_list', [1, 2, 3], ttl=60)
        res = client.get('test_list')
        assert res and res == [1, 2, 3]
        assert isinstance(res, list)


if __name__ == '__main__':
    test_auto_get_dict()
    test_auto_get_list()


import json

from datetime import datetime, date


class DateTimeEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, (date, datetime)):
            return obj.isoformat()
        return super().default(obj)



import json
from contextlib import asynccontextmanager
from datetime import date, datetime
from typing import Any

from .sync_client import RedisClient
from .conf import RedisConfig
from .decorator import is_connected_async

import redis.asyncio as redis


class AsyncRedisClient(RedisClient):
    def __init__(self, cfg: RedisConfig | None = None):
        self.client: redis.Redis | None = None
        self.cfg: RedisConfig | None = cfg

    async def connect(self, cfg: RedisConfig | None = None):
        self.client = redis.Redis(
            host=cfg.host,
            port=cfg.port,
            password=cfg.password,
            decode_responses=True,
            db=cfg.db,
            client_name=cfg.client_name
        )
        return self.client

    async def close(self):
        await self.client.close()

    @is_connected_async
    async def get_key(self, key: str) -> Any:
        value = await self.client.get(key)
        if value is not None:
            try:
                # Пробуем десериализовать JSON строку
                json_value = json.loads(value)
                return self._deserialize(json_value)
            except json.JSONDecodeError:
                # Если это не JSON, проверяем на дату/время
                return self._deserialize(value)
        return value

    @is_connected_async
    async def set_key(self, key: str, value: Any, ttl: int = None):
        if isinstance(value, (dict, list, tuple, set, date, datetime)):
            value = self.encoder.encode(value)
        await self.client.set(key, value, ex=ttl)

    @is_connected_async
    async def remove_key(self, key: str):
        await self.client.delete(key)

    def _serialize_to_json_string(self, value: Any) -> str:
        """Сериализация в JSON строку для Redis"""
        if isinstance(value, (dict, list, tuple, set, date, datetime)):
            return self.encoder.encode(value)  # Возвращает строку
        return str(value)

    @is_connected_async
    async def set_json_str_key(self, key: str, value: Any, ttl: int = None):
        serialized_value = self._serialize_to_json_string(value)
        await self.client.set(key, serialized_value)
        if ttl:
            await self.client.expire(key, ttl)

    @is_connected_async
    async def get_json_str_key(self, key: str) -> Any:
        result = await self.client.get(key)
        if result:
            try:
                json_data = json.loads(result)
                return self._deserialize(json_data)
            except json.JSONDecodeError:
                return self._deserialize(result)
        return result

    @is_connected_async
    async def set_json_key(self, key: str, value: Any, ttl: int = None):
        """Установить JSON через RedisJSON (JSON.SET команда)"""
        # RedisJSON ожидает объект Python (dict/list), не строку
        await self.client.json().set(key, "$", value)
        if ttl:
            await self.client.expire(key, ttl)

    @is_connected_async
    async def get_json_key(self, key: str) -> Any:
        """Получить JSON через RedisJSON (JSON.GET команда)"""
        result = await self.client.json().get(key)
        if result is not None:
            return self._deserialize(result)
        return result

    @is_connected_async
    async def get_key_type(self, key: str) -> str:
        return await self.client.type(key)

    @is_connected_async
    async def exists(self, key: str) -> bool:
        return await self.client.exists(key)

    @is_connected_async
    async def set_ttl(self, key: str, ttl: int):
        await self.client.expire(key, ttl)

    @is_connected_async
    async def get_ttl(self, key: str) -> int:
        return await self.client.ttl(key)

    @is_connected_async
    async def multiple_set(self, data: dict[str, Any], ttl: int = None):
        await self.client.mset(data)
        if ttl:
            for k in data.keys():
                await self.set_ttl(k, ttl)

    @is_connected_async
    async def multiple_get(self, *args):
        values = await self.client.mget(*args)
        return dict(zip(args, values))

    @asynccontextmanager
    async def connected(self, cfg: RedisConfig):
        self.client = await self.connect(cfg)
        try:
            yield self
        finally:
            await self.close()

    async def __aenter__(self):
        if self.client is None and self.cfg is not None:
            await self.connect(self.cfg)
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()

    @is_connected_async
    def set(self, key: str, value: Any, ttl: int = None):
        if isinstance(value, (dict, list, tuple, set)):
            self.set_json_key(key, value, ttl)
        else:
            self.set_key(key, value, ttl)

    @is_connected_async
    def get(self, key: str) -> Any:
        try:
            value = await self.get_json_key(key)
        except redis.exceptions.ResponseError:
            value = self.get_key(key)
        return value



import json
import re
from contextlib import contextmanager
from datetime import date, datetime
from typing import Any

import redis

from .decorator import is_connected
from .conf import RedisConfig
from .encoder import DateTimeEncoder


class RedisClient:
    encoder = DateTimeEncoder()
    iso_date_pattern = re.compile(
        r'^\d{4}-\d{2}-\d{2}(T\d{2}:\d{2}:\d{2}(\.\d+)?(Z|[+-]\d{2}:\d{2})?)?$'
    )

    def __init__(self, cfg: RedisConfig | None = None):
        self.client: redis.Redis | None = (self.connect(cfg) if cfg else None)
        self.cfg: RedisConfig | None = cfg

    def connect(self, cfg: RedisConfig):
        self.client = redis.Redis(
            host=cfg.host,
            port=cfg.port,
            password=cfg.password,
            decode_responses=True,
            db=cfg.db,
            client_name=cfg.client_name
        )
        return self.client

    @is_connected
    def close(self):
        self.client.close()

    def _serialize(self, value: Any) -> Any:
        if isinstance(value, (dict, list, tuple, set, date, datetime)):
            return json.loads(self.encoder.encode(value))
        return value

    def _deserialize(self, data: Any) -> Any:
        if isinstance(data, str) and self.iso_date_pattern.match(data):
            try:
                if 'T' in data:
                    return datetime.fromisoformat(data.replace('Z', '+00:00'))
                return date.fromisoformat(data)
            except ValueError:
                return data
        elif isinstance(data, dict):
            return {k: self._deserialize(v) for k, v in data.items()}
        elif isinstance(data, list):
            return [self._deserialize(item) for item in data]
        return data

    @is_connected
    def get_json_key(self, key: str) -> Any:
        result = self.client.json().get(key)
        return self._deserialize(result)

    @is_connected
    def set_json_key(self, key: str, value: Any, ttl: int = None):
        serialized_value = self._serialize(value)
        self.client.json().set(key, "$", serialized_value)
        if ttl:
            self.set_ttl(key, ttl)

    @is_connected
    def get_key(self, key: str) -> Any:
        value = self.client.get(key)
        if value is not None:
            try:
                # Пробуем десериализовать JSON строку
                json_value = json.loads(value)
                return self._deserialize(json_value)
            except json.JSONDecodeError:
                # Если это не JSON, проверяем на дату/время
                return self._deserialize(value)
        return value

    @is_connected
    def set_key(self, key: str, value: Any, ttl: int = None):
        if isinstance(value, (dict, list, tuple, set, date, datetime)):
            value = self.encoder.encode(value)
        return self.client.set(key, value, ex=ttl)

    @is_connected
    def remove_key(self, key: str):
        self.client.delete(key)

    @is_connected
    def get_key_type(self, key: str) -> str:
        return self.client.type(key)

    @is_connected
    def exists(self, key: str) -> bool:
        return self.client.exists(key)

    @is_connected
    def set_ttl(self, key: str, ttl: int):
        self.client.expire(key, ttl)

    @is_connected
    def get_ttl(self, key: str) -> int:
        return self.client.ttl(key)

    @is_connected
    def multiple_set(self, data: dict[str, Any], ttl: int = None):
        self.client.mset(data)
        for k in data.keys():
            self.set_ttl(k, ttl)

    @is_connected
    def multiple_get(self, *args):
        values = self.client.mget(*args)
        return dict(zip(args, values))

    @contextmanager
    def connected(self, cfg: RedisConfig):
        self.client = self.connect(cfg)
        try:
            yield self
        finally:
            self.close()

    def __enter__(self):
        if self.client is None and self.cfg is not None:
            self.connect(self.cfg)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    @is_connected
    def set(self, key: str, value: Any, ttl: int = None):
        if isinstance(value, (dict, list, tuple, set)):
            self.set_json_key(key, value, ttl)
        else:
            self.set_key(key, value, ttl)

    @is_connected
    def get(self, key: str) -> Any:
        try:
            value = self.get_json_key(key)
        except redis.exceptions.ResponseError:
            value = self.get_key(key)
        return value



def is_connected(func):
    def wrapper(*args, **kwargs):
        self = args[0]
        if self.client is None:
            raise RuntimeError('Client is not connected')
        return func(*args, **kwargs)
    return wrapper


def is_connected_async(func):
    async def wrapper(*args, **kwargs):
        self = args[0]
        if self.client is None:
            raise RuntimeError('Client is not connected')
        return await func(*args, **kwargs)
    return wrapper



from dgredis import RedisConfig, RedisClient


def test_remove_simple_key():
    rconf = RedisConfig(
        host='localhost',
        port=6379,
        db=1
    )
    with RedisClient().connected(cfg=rconf) as client:
        client.set_key('test', 'this is a test 2 var')
        client.remove_key('test')
        assert client.get_key('test') is None


def test_remove_json_key():
    rconf = RedisConfig(
        host='localhost',
        port=6379,
        db=1
    )
    with RedisClient().connected(cfg=rconf) as client:
        client.set_json_key('test_json', {"value": "this is a test 2 var"})
        client.remove_key('test_json')
        assert client.get_key('test_json') is None


if __name__ == '__main__':
    test_remove_simple_key()
    test_remove_json_key()



from dgredis import RedisClient, RedisConfig
from time import sleep


def test_context_manager():
    rconf = RedisConfig(
        host='localhost',
        port=6379,
        db=1,
        client_name='dgredis_test'
    )
    with RedisClient().connected(cfg=rconf) as client:
        client.set_key('test', 'this is a context manager test 1', ttl=60)
        assert client.get_key('test') == 'this is a context manager test 1'
        print("assert true")


def test_context_manager2():
    rconf = RedisConfig(
        host='localhost',
        port=6379,
        db=1,
        client_name='dgredis_test'
    )
    with RedisClient(cfg=rconf) as client:
        client.set_key('test', 'this is a context manager test 2', ttl=60)
        assert client.get_key('test') == 'this is a context manager test 2'
        print("assert true")


if __name__ == '__main__':
    test_context_manager()
    test_context_manager2()



import asyncio

from dgredis import RedisConfig, AsyncRedisClient


async def test_async_client():
    rconf = RedisConfig(
        host="localhost",
        port=6379,
        db=1,
        client_name="dgredis_test"
    )
    async with AsyncRedisClient(cfg=rconf) as client:
        await client.set_key("test_async", "this is a async client test", ttl=60)
        assert await client.get_key("test_async") == "this is a async client test"
        print("assert true")


async def test_async_json_str():
    rconf = RedisConfig(
        host="localhost",
        port=6379,
        db=1,
        client_name="dgredis_test"
    )
    async with (AsyncRedisClient(cfg=rconf) as client):
        await client.set_json_str_key("test_async_json", {"value": 'this is a async client test 2'}, ttl=60)
        v = await client.get_json_str_key("test_async_json")
        assert v and v.get("value") == "this is a async client test 2"
        print("assert true")


async def test_async_json_native():
    rconf = RedisConfig(
        host="localhost",
        port=6379,
        db=1,
        client_name="dgredis_test"
    )
    async with (AsyncRedisClient(cfg=rconf) as client):
        await client.set_json_key("test_async_json_native", {"value": 'this is a async client test 2'}, ttl=60)
        v = await client.get_json_key("test_async_json_native")
        assert v and v.get("value") == "this is a async client test 2"
        print("assert true")


if __name__ == '__main__':
    asyncio.run(test_async_client())
    asyncio.run(test_async_json_str())
    asyncio.run(test_async_json_native())



from setuptools import setup, find_packages


with open('README.md') as f:
    long_description = f.read()


setup(name='dgredis',
      version='0.1.1',
      description='Ver.0.1.0',
      long_description=long_description,
      long_description_content_type='text/markdown',  # This is important!
      classifiers=[
                   'Development Status :: 5 - Production/Stable',
                   #'Development Status :: 3 - Alpha',
                   'License :: OSI Approved :: MIT License',
                   'License :: OSI Approved :: Apache Software License',
                   'Programming Language :: Python :: 3',
                   "Operating System :: OS Independent",
                   ],
      keywords='',
      url='https://gitlab.com/gng-group/dgredis.git',
      author='Malanris',
      author_email='admin@roro.su',
      license='MIT',
      packages=find_packages(),
      install_requires=[
          "redis"
      ],
      include_package_data=True,
      zip_safe=False)



from dgredis.conf import RedisConfig
from dgredis import RedisClient


def test_db_index():
    rconf = RedisConfig(
        host='localhost',
        port=6379,
        db=1
    )
    client = RedisClient(cfg=rconf)
    client.set_key("test", "this is a test", ttl=60)
    assert client.get_key("test") == "this is a test"

if __name__ == '__main__':
    test_db_index()
